@@ -5,4 +5,4 @@ from .taskresult import TaskResult, TaskSeed
 MsgFields = _MsgFields()
 TaskResults = TaskResults
 TaskResult = TaskResult
-TaskResult = TaskSeed
+TaskSeed = TaskSeed
