@@ -1,9 +1,13 @@
 # -*- coding: utf-8 -*-
 
 import logging
+import colander
 
 from datetime import datetime
 from .fields import MsgFields as F
+from .exceptions import ValidationError
+import schemas
+
 
 SEED_VERSION = "9.9.9"
 
@@ -17,7 +21,7 @@ class TaskResult(dict):
     TaskResult represents a Task result for a single Target Monitoring Object
     (TMO). It's designed to be used inside task functions like this:
 
-    tr = TaskResult("mytask", 1111)
+    tr = TaskResult("mytask", "2.2.5", 1111)
     tr.add("analysis", [1, 2, 3])
     tr.finish()
 
@@ -26,17 +30,22 @@ class TaskResult(dict):
 
     reserved_keys = [F.META]
 
-    def __init__(self, task, task_version, tmoid, data=None):
+    def __init__(self, task_name, task_version, tmoid, data=None,
+                 schema=None):
         # str name of the task's output.
         # Not used internally, but referenced when adding to TaskResults.
-        self.task = task
+        self.task_name = task_name
         # Supplied by task.
         self.task_version = task_version
         # int with the id of the target monitor object
         # Not used internally, but referenced when adding to TaskResults.
         self.tmoid = tmoid
+        # Should be class of type colander.Schema.
+        # Chiefly for testing - by default, will look for corresponding
+        # schema to task_name
+        self.schema = self._set_schema(schema)
 
-        if not self.task:
+        if not self.task_name:
             raise ValueError("Must define a task type")
         if (not self.task_version or
                 not isinstance(self.task_version, basestring)):
@@ -57,7 +66,7 @@ class TaskResult(dict):
             self.update(data)
 
     @classmethod
-    def init_from_dict(cls, task, tmoid, data):
+    def init_from_dict(cls, task, tmoid, data, schema=None):
         """
         Load a TaskResult from a dict, likely a deserialized TaskResult from
         somewhere else. Note that task_version is contained in the meta data
@@ -72,7 +81,30 @@ class TaskResult(dict):
         except KeyError:
             raise ValueError("TaskResult dict does not define a version "
                              "number")
-        return cls(task, task_version, tmoid, data)
+        return cls(task, task_version, tmoid, data, schema=schema)
+
+    def _set_schema(self, supplied_schema):
+        schema_cls = None
+
+        if not self.task_name:
+            raise ValueError("Must specify a task_name")
+
+        print ("Supplied schema: %s" % supplied_schema)
+        if not supplied_schema:
+            print "No schema supplied, going auto"
+            schema_name = "{0}Schema".format(self.task_name.title())
+            try:
+                schema_cls = getattr(schemas, schema_name)
+            except AttributeError:
+                raise ValueError("Couldn't find validation schema called {0}"
+                                   .format(schema_name))
+        else:
+            schema_cls = supplied_schema
+        print (schema_cls)
+        if schemas._BaseTaskSchema not in schema_cls.__bases__:
+            raise ValueError("schema must be of type colander.Schema")
+
+        return schema_cls
 
     def add(self, result_name, result_val):
         """
@@ -96,7 +128,7 @@ class TaskResult(dict):
         except KeyError:
             logging.debug(self)
             raise ValueError("{0}.{1} has no value for {2}".format(
-                self.task, self.tmoid, key))
+                self.task_name, self.tmoid, key))
 
     def get_data(self, key):
         """
@@ -150,6 +182,15 @@ class TaskResult(dict):
         self[F.META][F.META_END_TIME] = self.time_end
         self[F.META][F.META_VERSION] = self.task_version
 
+        self.schema_validate()
+
+    def schema_validate(self):
+        try:
+            self.schema().deserialize(self)
+        except colander.Invalid as e:
+            raise ValidationError(
+                "Couldn't validate schema: {0}".format(e))
+
 
 class TaskSeed(TaskResult):
     def __init__(self, task, tmoid, key, val):
