@@ -11,14 +11,14 @@ class TaskResults(dict):
         dict.__init__(self, *args, **kwargs)
 
     def add_task_result(self, task_result):
-        if not task_result.task:
+        if not task_result.task_name:
             raise ValueError("TaskResult must specify a task")
         if not task_result.tmoid:
             raise ValueError("TaskResult must specify a target monitoring "
                              "object id")
 
-        self.setdefault(task_result.task, {})
-        self[task_result.task][task_result.tmoid] = dict(task_result)
+        self.setdefault(task_result.task_name, {})
+        self[task_result.task_name][task_result.tmoid] = dict(task_result)
 
     def tasks(self):
         return self.iterkeys()
@@ -31,7 +31,7 @@ class TaskResults(dict):
 
         return self[task].iterkeys()
 
-    def get_task_result(self, task, tmoid):
+    def get_task_result(self, task, tmoid, schema=None):
         """
         Get a single TaskResult based on a task name and target monitoring
         ID.
@@ -42,5 +42,5 @@ class TaskResults(dict):
             raise ValueError("Could not find task {0} with tmoid {1} in "
                              "results".format(task, tmoid))
 
-        tr = TaskResult.init_from_dict(task, tmoid, self[task][tmoid])
+        tr = TaskResult.init_from_dict(task, tmoid, self[task][tmoid], schema=schema)
         return tr
