@@ -2,17 +2,20 @@
 
 import unittest
 import json
+import colander
 
 from ..taskresult import TaskResult, TaskSeed, SEED_VERSION
 from ..fields import MsgFields as F
+from ..exceptions import ValidationError
+from ..schemas import _BaseTaskSchema
 
 
 class TestTaskResult(unittest.TestCase):
     def setUp(self):
-        pass
+        self.task_name = "test"
 
     def test_init(self):
-        m = TaskResult("mytask", "1.1.1", 111)
+        m = TaskResult(self.task_name, "1.1.1", 111)
         self.assertIsNotNone(m)
 
     def test_missing_taskid(self):
@@ -28,7 +31,7 @@ class TestTaskResult(unittest.TestCase):
     def test_missing_task_version(self):
         tr = None
         try:
-            tr = TaskResult("mytask", None, 111)
+            tr = TaskResult(self.task_name, None, 111)
         except ValueError:
             assert(True)
         else:
@@ -38,7 +41,7 @@ class TestTaskResult(unittest.TestCase):
     def test_non_str_task_version(self):
         tr = None
         try:
-            tr = TaskResult("mytask", 1.1, 111)
+            tr = TaskResult(self.task_name, 1.1, 111)
         except ValueError:
             assert(True)
         else:
@@ -48,7 +51,7 @@ class TestTaskResult(unittest.TestCase):
     def test_missing_tmoid(self):
         tr = None
         try:
-            tr = TaskResult("mytask", "1.1.1", None)
+            tr = TaskResult(self.task_name, "1.1.1", None)
         except ValueError:
             assert(True)
         else:
@@ -58,7 +61,7 @@ class TestTaskResult(unittest.TestCase):
     def test_non_numeric_tmoid(self):
         tr = None
         try:
-            tr = TaskResult("mytask", "1.1.1", "abc")
+            tr = TaskResult(self.task_name, "1.1.1", "abc")
         except ValueError:
             assert(True)
         else:
@@ -66,25 +69,25 @@ class TestTaskResult(unittest.TestCase):
         self.assertIsNone(tr)
 
     def test_add(self):
-        tr = TaskResult("mytask", "1.1.1", 38974823)
-        tr.add("results", "Here are my results")
-        self.assertEqual(tr.get_data("results"), "Here are my results")
-        self.assertEqual(tr["results"], "Here are my results")
+        tr = TaskResult(self.task_name, "1.1.1", 38974823)
+        tr.add("a", "Here are my results")
+        self.assertEqual(tr.get_data("a"), "Here are my results")
 
+    @unittest.skip
     def test_add_nested_data(self):
-        tr = TaskResult("mytask", "1.1.1", 10098)
+        tr = TaskResult(self.task_name, "1.1.1", 10098)
         tr.add("results", {"a": "Nested"})
         self.assertEqual(tr.get_data("results"), {"a": "Nested"})
         self.assertEqual(tr["results"], {"a": "Nested"})
 
     def test_missing_data(self):
-        tr = TaskResult("mytask", "1.1.1", 10098)
-        tr.add("results", {"a": "Nested"})
+        tr = TaskResult(self.task_name, "1.1.1", 10098)
+        tr.add("a", "Here are my results")
         self.assertIsNone(tr.get_data("nothere"))
 
     def test_missing_data_strict(self):
-        tr = TaskResult("mytask", "1.1.1", 10098)
-        tr.add("results", {"a": "Nested"})
+        tr = TaskResult(self.task_name, "1.1.1", 10098)
+        tr.add("a", "Here are my results")
         try:
             tr.get_data_strict("nothere")
         except ValueError:
@@ -93,34 +96,43 @@ class TestTaskResult(unittest.TestCase):
             assert(False)
 
     def test_dict_loop(self):
-        tr = TaskResult("mytask", "1.1.1", 8734)
+        tr = TaskResult(self.task_name, "1.1.1", 8734)
         tr.add("a", "1")
         tr.add("b", "2")
         self.assertEqual(set(tr.keys()), set(["a", "b"]))
 
+    def test_multi_key(self):
+        tr = TaskResult(self.task_name, "1.1.1", 8734)
+        tr.add("a", "1")
+        tr.add("b", "2")
+        tr.finish()
+
+        self.assertEqual(tr.get_data("a"), "1")
+        self.assertEqual(tr.get_data("b"), "2")
+
     def test_finalise_success(self):
-        tr = TaskResult("mytask", "1.1.1", 123)
-        tr.add("results", "result")
+        tr = TaskResult(self.task_name, "1.1.1", 123)
+        tr.add("a", "result")
         tr.finish()
         self.assertEqual(tr.success, True)
 
     def test_finalise_failure(self):
-        tr = TaskResult("mytask", "1.1.1", 123)
-        tr.add("results", "result")
+        tr = TaskResult(self.task_name, "1.1.1", 123)
+        tr.add("a", "result")
         tr.finish(err="Everything broken")
         self.assertFalse(tr.success)
         self.assertIsNotNone(tr.err)
 
     def test_timer_success(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
-        tr.add("whois", "My whois data")
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("a", "My whois data")
         tr.finish()
         self.assertIsNotNone(tr.time_end)
         self.assertLess(tr.time_start, tr.time_end)
 
     def test_timer_fail(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
-        tr.add("whois", "My whois data")
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("a", "My whois data")
         tr.finish(err="Everything broke")
         self.assertIsNotNone(tr.time_end)
         self.assertLess(tr.time_start, tr.time_end)
@@ -128,27 +140,27 @@ class TestTaskResult(unittest.TestCase):
         self.assertEqual(tr[F.META][F.META_END_TIME], tr.time_end)
 
     def test_success(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
-        tr.add("whois", "My whois data")
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("a", "My whois data")
         tr.finish()
         self.assertTrue(tr.success)
         self.assertIsNone(tr.err)
 
     def test_failure(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
-        tr.add("whois", "My whois data")
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("a", "My whois data")
         tr.finish(err="Everything broke")
         self.assertFalse(tr.success)
         self.assertEqual(tr.err, "Everything broke")
 
     def test_msg(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
-        tr.add("whois", "My whois data")
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("a", "My whois data")
         tr.finish(msg="Diagnostic info")
         self.assertEqual(tr[F.META][F.META_MSG], "Diagnostic info")
 
     def test_reserved_name(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
+        tr = TaskResult(self.task_name, "1.1.1", 100)
         try:
             tr.add(F.META, "Reserved field")
         except ValueError:
@@ -157,36 +169,73 @@ class TestTaskResult(unittest.TestCase):
             assert(False)
 
     def test_version(self):
-        tr = TaskResult("mytask", "3.4.5", 100)
-        tr.add("task", "result")
+        tr = TaskResult(self.task_name, "3.4.5", 100)
+        tr.add("a", "result")
         tr.finish()
 
         self.assertEqual(tr.version, "3.4.5")
 
     def test_obj_json_import(self):
-        tr = TaskResult("mytask", "1.1.1", 100)
-        tr.add("whois", "My whois data")
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("a", "My whois data")
         tr.finish(err="Everything broke")
 
         tr_json = json.dumps(tr)
 
-        tr2 = TaskResult.init_from_dict("mytask", 100, json.loads(tr_json))
-        self.assertEqual(tr2.get_data("whois"), "My whois data")
+        tr2 = TaskResult.init_from_dict(self.task_name,
+                                        100, json.loads(tr_json))
+        self.assertEqual(tr2.get_data("a"), "My whois data")
         self.assertFalse(tr2.success)
         self.assertEqual(tr2.err, "Everything broke")
         self.assertEqual(tr2.task_version, "1.1.1")
 
+    def test_schema_fail(self):
+        tr = TaskResult(self.task_name, "1.1.1", 100)
+        tr.add("nosuch", "My whois data")
+        try:
+            tr.finish()
+        except ValidationError:
+            assert(True)
+        else:
+            assert(False)
+
+    def test_custom_schema(self):
+        class TestCustomSchema(_BaseTaskSchema):
+            z = colander.SchemaNode(colander.String(), name="z")
+        tr = TaskResult(self.task_name, "1.1.1", 100, schema=TestCustomSchema)
+        tr.add("z", "My whois data")
+        tr.finish()
+
+        self.assertEqual(tr.get_data("z"), "My whois data")
+
+    def test_custom_schema_invalid(self):
+        class TestCustomInvalidSchema(_BaseTaskSchema):
+            z = colander.SchemaNode(colander.String(), name="z")
+
+        tr = TaskResult(self.task_name, "1.1.1", 100,
+                        schema=TestCustomInvalidSchema)
+        tr.add("y", "My whois data")
+        try:
+            tr.finish()
+        except ValidationError:
+            assert(True)
+        else:
+            assert(False)
+
 
 class TestTaskSeed(unittest.TestCase):
+    def setUp(self):
+        self.task_name = "test"
+
     def test_taskseed_payload(self):
-        ts = TaskSeed("mytask", 100, "hostname", "example.com")
-        self.assertEqual(ts.get_data("hostname"), "example.com")
+        ts = TaskSeed(self.task_name, 100, "a", "example.com")
+        self.assertEqual(ts.get_data("a"), "example.com")
 
     def test_taskseed_version(self):
-        ts = TaskSeed("mytask", 100, "hostname", "example.com")
+        ts = TaskSeed(self.task_name, 100, "a", "example.com")
         self.assertEqual(ts.version, SEED_VERSION)
 
     def test_taskseed_dates(self):
-        ts = TaskSeed("mytask", 100, "hostname", "example.com")
+        ts = TaskSeed(self.task_name, 100, "a", "example.com")
         self.assertIsNotNone(ts.get_meta(F.META_START_TIME))
         self.assertIsNotNone(ts.get_meta(F.META_END_TIME))
